from abc import ABC, abstractmethod
from ..models import ResponseModel
from typing import TYPE_CHECKING, Any, Dict, Optional, Sequence, List
from unionllm.utils import ModelResponse, Message, Choices, Usage, Context, StreamingChoices, Delta

import openai
import json
from openai._models import BaseModel as OpenAIObject

# if TYPE_CHECKING:
from dataclasses import dataclass

@dataclass
class BaseProvider(ABC):
    args: Optional[Dict[str, Any]] = None
    key: Optional[str] = None
    group: Optional[str] = None

    def __init__(
        self,
        status_code,
        message,
    ):
        self.status_code = status_code
        self.message = message

    @abstractmethod
    def completion(self, model: str, messages: list) -> ResponseModel:
        pass

    def create_model_response(
        self, openai_response: openai.ChatCompletion, model: str
    ) -> ModelResponse:
        # print("openai_response: ", openai_response)
        choices = []

        for choice in openai_response.choices:
            message = Message(content=choice.message.content, role=choice.message.role)
            choices.append(
                Choices(
                    message=message, index=choice.index, finish_reason=choice.finish_reason
                )
            )

        usage = Usage(
            prompt_tokens=openai_response.usage.prompt_tokens,
            completion_tokens=openai_response.usage.completion_tokens,
            total_tokens=openai_response.usage.total_tokens,
        )
        response = ModelResponse(
            id=openai_response.id,
            choices=choices,
            created=openai_response.created,
            model=model,
            usage=usage,
        )
        return response
    
    def post_stream_processing(self, response, model=None):
        for chunk in response:
            data = chunk.json()
            if isinstance(data, str):
                data = json.loads(data)
            if 'choices' in data:
                choices = data['choices']
                chunk_choices = []
                for choice in choices:
                    delta = choice.get("delta")
                    if delta:
                        chunk_delta = Delta()
                        if "role" in choice['delta']:
                            chunk_delta.role = choice['delta']["role"]
                        if "content" in choice['delta']:
                            chunk_delta.content = choice['delta']["content"]
                        chunk_choices.append(StreamingChoices(index=choice['index'], delta=chunk_delta))

            if "usage" in data:
                chunk_usage = Usage()
                if data["usage"]:
                    if "prompt_tokens" in data["usage"]:
                        chunk_usage.prompt_tokens = data["usage"]["prompt_tokens"]
                    if "completion_tokens" in data["usage"]:
                        chunk_usage.completion_tokens = data["usage"]["completion_tokens"]
                    if "total_tokens" in data["usage"]:
                        chunk_usage.total_tokens = data["usage"]["total_tokens"]
            
            chunk_response = ModelResponse(
                id=data["id"],
                choices=chunk_choices,
                created=data["created"],
                model=model,
                usage=chunk_usage if "usage" in data else None,
                stream=True
            )
            # print(chunk_response)
            yield chunk_response


class UnionLLMError(Exception):
    """Base exception class for UnionLLM."""
    pass

class ProviderError(UnionLLMError):
    """Exception raised when there is an issue with the model provider."""
    pass

class APICallError(UnionLLMError):
    """Exception raised when an API call fails."""
    pass


import sys
import os
import json
import pytest
import litellm
from dotenv import load_dotenv
load_dotenv()

# 将项目根目录添加到sys.path中
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from unionllm.providers.litellm import LiteLLMProvider, LiteLLMError
# 将项目根目录添加到sys.path中
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from unionllm.providers.base_provider import BaseProvider

class TestLiteLLMProvider:
    @pytest.fixture(autouse=True)
    def setup_provider(self):
        # 从环境变量中导入API密钥
        self.provider = LiteLLMProvider()
    
    def test_completion(self):
        # provider = "openai"
        model = "gpt-3.5-turbo"
        messages = [{"content": "Introduce yourself briefly", "role": "user"}]
        kwargs = {
            # "provider": provider,
            "model": model,
            "temperature": 0.5,
            "api_key": os.getenv("OPENAI_API_KEY")
        }
        response = self.provider.completion(model=model, messages=messages, stream=False)
        print("response: ", response)


import logging
from typing import Any, List
from .providers import zhipu, moonshot, minimax, qwen, tiangong, baichuan, wenxin, xunfei, dify, fastgpt, coze, litellm
from .exceptions import ProviderError
from litellm import completion as litellm_completion

logger = logging.getLogger(__name__)

class UnionLLM:
    def __init__(self, provider: str, **kwargs):
        self.provider = provider.lower()
        self.litellm_call_type = None
        if self.provider == "zhipuai":
            self.provider_instance = zhipu.ZhipuAIProvider(**kwargs)
        elif self.provider == "moonshot":
            self.provider_instance = moonshot.MoonshotAIProvider(**kwargs)
        elif self.provider == "minimax":
            self.provider_instance = minimax.MinimaxAIProvider(**kwargs)
        elif self.provider == "qwen":
            self.provider_instance = qwen.QwenAIProvider(**kwargs)
        elif self.provider == "tiangong":
            self.provider_instance = tiangong.TianGongAIProvider(**kwargs)
        elif self.provider == "baichuan":
            self.provider_instance = baichuan.BaiChuanAIProvider(**kwargs)
        elif self.provider == "wenxin":
            self.provider_instance = wenxin.WenXinAIProvider(**kwargs)
        elif self.provider == "xunfei":
            self.provider_instance = xunfei.XunfeiAIProvider(**kwargs)
        elif self.provider == "dify":
            self.provider_instance = dify.DifyAIProvider(**kwargs)
        elif self.provider == "fastgpt":
            self.provider_instance = fastgpt.FastGPTProvider(**kwargs)
        elif self.provider == "coze":
            self.provider_instance = coze.CozeAIProvider(**kwargs)
        elif self.provider:
            if_litellm_support, support_type = self.check_litellm_providers(provider=self.provider)
            if if_litellm_support:
                self.provider_instance = litellm.LiteLLMProvider(**kwargs)
                if support_type == 1:
                    self.litellm_call_type = 1
                elif support_type == 2:
                    self.litellm_call_type = 2
            else:
                raise ProviderError(f"Provider '{self.provider}' is not supported.")
        else:
            self.provider_instance = litellm.LiteLLMProvider(**kwargs)

    def completion(self, model: str, messages: List[str], **kwargs) -> Any:
        if not self.provider_instance:
            raise ProviderError(f"Provider '{self.provider}' is not initialized.")
        
        if self.litellm_call_type:
            if self.litellm_call_type == 1:
                # Jugde whether the model starts with self.provider, if not, add it
                if not model.startswith(self.provider+"/"):
                    model = f"{self.provider}/{model}"
                return self.provider_instance.completion(model, messages, **kwargs)
            elif self.litellm_call_type == 2:
                return self.provider_instance.completion(model, messages, **kwargs)
        else:
            return self.provider_instance.completion(model, messages, **kwargs)
        
    def check_litellm_providers(self, provider: str) -> bool:
        # Judge whether the provider is supported by LiteLLM, and if provider name should be added to the model name
        if provider in ['azure', 'sagemaker', 'bedrock', 'vertex_ai', 'palm', 'gemini', 'mistral', 'cloudflare', 'huggingface', 'replicate', 'together_ai', 'openrouter', 'baseten', 'nlp_cloud', 'petals', 'ollama', 'perplexity', 'groq', 'anyscale', 'watsonx', 'voyage', 'xinference']:
            # provider name should be added to the model name
            return True, 1
        elif provider in ['openai', 'claude', 'cohere', 'ai21', 'luminous']:
            # provider name should not be added to the model name
            return True, 2
        else:
            return False, 0
        
# This is the new function to simplify calling
def unionchat(model: str, messages: List[dict], **kwargs) -> Any:
    client = UnionLLM(**kwargs)
    return client.completion(model=model, messages=messages, **kwargs)


import sys
import os
import json
import pytest
from dotenv import load_dotenv
load_dotenv()

# 将项目根目录添加到sys.path中
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from unionllm.providers.xunfei import XunfeiAIProvider, XunfeiOpenAIError

class TestXunfeiAIProvider:
    @pytest.fixture(autouse=True)
    def setup_provider(self):
        # 从环境变量中导入API密钥
        app_id=os.getenv("XUNFEI_APP_ID")
        self.provider = XunfeiAIProvider(app_id=os.getenv("XUNFEI_APP_ID"), api_key=os.getenv("XUNFEI_API_KEY"), api_secret=os.getenv("XUNFEI_API_SECRET"))

    def test_completion_non_stream(self):
        # 定义模型和消息
        model = "generalv3"
        messages = [{"content": "introduce yourself briefly.", "role": "user"}]
        try:
            response = self.provider.completion(model=model, messages=messages, stream=False)
            print(response)
        except Exception as e:
            pytest.fail(f"Error occurred: {e}")



import sys
import os
import json
import pytest
from dotenv import load_dotenv
load_dotenv()

# 将项目根目录添加到sys.path中
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from unionllm.providers.moonshot import MoonshotAIProvider, MoonshotOpenAIError

class TestMoonshotAIProvider:
    @pytest.fixture(autouse=True)
    def setup_provider(self):
        # 从环境变量中导入API密钥
        self.provider = MoonshotAIProvider(api_key=os.getenv("MOONSHOT_API_KEY"))

    def test_completion_stream(self):
        # Test non-stream completion
        model = "moonshot-v1-8k"
        messages = [{"content": "introduce yourself briefly.", "role": "user"}]        
        try:
            response = self.provider.completion(model=model, messages=messages, stream=True)
            for chunk in response:
                print(chunk["choices"][0]["delta"]["content"])
        except Exception as e:
            pytest.fail(f"Error occurred: {e}")
        

    def test_completion_non_stream(self):
        # 定义模型和消息
        model = "moonshot-v1-8k"
        messages = [{"content": "introduce yourself briefly.", "role": "user"}]
        try:
            response = self.provider.completion(model=model, messages=messages, stream=False)
            print(response)
        except Exception as e:
            pytest.fail(f"Error occurred: {e}")
